from django.db.models import Q
from django.utils.translation import get_language
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.conf import settings

from rest_framework import viewsets, filters
from rest_framework.permissions import IsAuthenticated

from django_filters.rest_framework import DjangoFilterBackend

from django_ical.views import ICalFeed

from .models import Event

from .serializers import EventSerializer

User = get_user_model()


def get_queryset(queryset, user):
    # Show events from courses that the user is coordinator or assistant
    course_coord_query = Q(
        course__course_professors__user=user,
        # course__course_professors__role='coordinator'
    )
    # Show events from courses that the user is assistant
    course_assistants_query = Q(klass__assistants=user)

    current_language = get_language()
    # Show events from courses that the user is enrolld
    course_students_query = Q(
        Q(
            course__status='published',
            course__lang__contains=[current_language],
            course__course_students__user=user
        ) &
        Q(
            Q(course__is_public=True, ) |
            Q(course__groups__in=user.groups.all(), course__status='published')
        )
    )

    # General and corse start events must be show for all
    general_events_query = Q(
        Q(type='general') |
        Q(type='course_start')
    )

    queryset = queryset.filter(
        Q(
            course_coord_query |
            course_assistants_query |
            course_students_query |
            general_events_query
        )
    )
    return queryset


class EventViewSet(viewsets.ModelViewSet):

    queryset = Event.objects.all()
    serializer_class = EventSerializer
    permission_classes = [IsAuthenticated]
    filter_backends = [
        DjangoFilterBackend,
        filters.OrderingFilter,
        filters.SearchFilter,
    ]
    filterset_fields = ['course', 'type', 'klass',]
    ordering = ['-date_time',]
    ordering_fields = ['date_time',]
    search_fields = ['title', 'text',]

    def get_queryset(self):
        queryset = super().get_queryset()

        if not self.request.user.is_superuser:
            queryset = get_queryset(queryset, self.request.user)
        
        return queryset.distinct()


class EventFeed(ICalFeed):

    timezone = 'UTC'
    file_name = "calendar.ics"

    def get_object(self, request):

        current_site = Site.objects.get_current()
        self.product_id = current_site.domain

        if not request.user.is_anonymous:
            user = request.user
        else:
            user_id = request.GET.get('user','')
            if user_id:
                try:
                    user = User.objects.get(id=user_id)
                except User.DoesNotExist:
                    pass
            else:
                # If the user is anonymous and no user is passed, return empty
                return Event.objects.none()

        queryset = get_queryset(Event.objects.all(), user)

        course = request.GET.get('course','')
        if course:
            queryset = queryset.filter(course=course)

        return queryset.order_by('-date_time').distinct()

    def title(self):

        lang = get_language()

        if lang == 'en':
            return settings.SITE_NAME_EN
        if lang == 'es':
            return settings.SITE_NAME_ES
        if lang == 'pt-br':
            return settings.SITE_NAME_PT_BR
        return settings.SITE_NAME

    def items(self, obj):
        """
        Takes the object returned by get_object() and returns a list of
        items to publish in this feed.
        """
        return obj.all()

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.text

    def item_date_time(self, item):
        return item.date_time

    def item_start_datetime(self, item):
        return item.date_time

    def item_link(self, item):
        if item.course:
            return '/course/{}/intro/'.format(item.course.slug)
        else:
            return ''

    def item_guid(self, item):
        return str(item.date_time) + '@' + self.product_id