from django.shortcuts import get_object_or_404
from django.http import HttpResponse
from django.contrib.contenttypes.models import ContentType

from rest_framework import viewsets
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticated
from rest_framework.decorators import action

from docx import Document

from courses.models import Course, Unit
from courses.notes.models import Note
from courses.notes.serializers import (
    NoteSerializer,
    UnitNoteSerializer,
    LessonNoteSerializer,
    CourseNoteSerializer,
)


class NotesViewSet(viewsets.ModelViewSet):
    model = Note
    serializer_class = NoteSerializer
    filter_fields = ('content_type', 'object_id')
    permission_classes = (IsAuthenticated,)

    def perform_create(self, serializer):
        # Only create a new note if there isn't one already for this user in this unit
        if not Note.objects.filter(object_id=serializer.validated_data['object_id'],
                                   user=self.request.user).exists():
            serializer.save(user=self.request.user)

    def perform_update(self, serializer):
        serializer.save(user=self.request.user)

    def pre_save(self, obj):
        obj.user = self.request.user
        return super(NotesViewSet, self).pre_save(obj)

    def get_queryset(self):
        user = self.request.user
        return Note.objects.filter(user=user)


class UserNotesViewSet(viewsets.ReadOnlyModelViewSet):

    model = Course
    queryset = Course.objects.all()
    lookup_field = 'course'
    permission_classes = (IsAuthenticated,)

    @staticmethod
    def get_user_notes_by_course(user):
        """
        Return the user notes organized by course
        """
        units = Unit.objects.filter(notes__user=user).exclude(notes__isnull=True)
        courses = {}
        for unit in units:
            course = unit.lesson.course
            lesson = unit.lesson
            if course.slug not in courses:
                courses[course.slug] = course
                courses[course.slug].lessons_dict = {}
            if lesson.slug not\
                    in courses[course.slug].lessons_dict:
                courses[course.slug].lessons_dict[lesson.slug] = lesson
                courses[course.slug].lessons_dict[lesson.slug].units_notes = []
            unit_type = ContentType.objects.get_for_model(unit)
            note = get_object_or_404(Note, user=user, content_type__pk=unit_type.id, object_id=unit.id)
            unit.user_note = note
            courses[course.slug].lessons_dict[lesson.slug].units_notes.append(unit)

        return courses

    def retrieve(self, request, *args, **kwargs):
        user = self.request.user
        if 'course' in self.kwargs:
            course = get_object_or_404(Course, slug=self.kwargs['course'])
            units = Unit.objects.filter(lesson__course=course, notes__user=user).exclude(notes__isnull=True)

            lessons_dict = {}
            for unit in units:
                lesson = unit.lesson
                if lesson.slug not in lessons_dict:
                    lessons_dict[lesson.slug] = lesson
                    lessons_dict[lesson.slug].units_notes = []
                unit_type = ContentType.objects.get_for_model(unit)
                note = get_object_or_404(Note, user=user, content_type__pk=unit_type.id, object_id=unit.id)
                unit.user_note = note
                lessons_dict[lesson.slug].units_notes.append(unit)

            results = []
            for lesson in lessons_dict.values():
                results.append(LessonNoteSerializer(lesson).data)
            return Response(results)

    def list(self, request, *args, **kwargs):
        user = self.request.user
        courses = self.get_user_notes_by_course(user)

        results = []
        for course in courses.values():
            course.lessons_notes = course.lessons_dict.values()
            course.course_notes_number = Unit.objects.filter(lesson__course=course, notes__user=user).exclude(notes__isnull=True).count()
            del course.lessons_dict
            results.append(CourseNoteSerializer(course).data)
        return Response(results)


    @action(detail=False, methods=['get'])
    def download(self, request):

        courses = self.get_user_notes_by_course(self.request.user)

        document = Document()

        # TODO Translate this
        document.add_heading('AEA Notes', 0)

        for course in courses.values():
            document.add_heading(course.name, level=1)
            for lesson in course.lessons_dict.values():
                document.add_heading(lesson.name, level=2)
                for unit in lesson.units_notes:
                    document.add_heading(unit.title, level=3)
                    document.add_paragraph(unit.user_note.text)

        response = HttpResponse(content_type='application/vnd.openxmlformats-officedocument.wordprocessingml.document')
        response['Content-Disposition'] = 'attachment; filename=my-notes.docx'
        document.save(response)

        return response
