from django.db.models.signals import pre_save, post_save, post_delete
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import gettext as _
from django.dispatch import receiver

from allauth.account.signals import email_confirmed

from .models import Course, Lesson, Unit, ProfessorMessage
from .tasks import publish_lesson, publish_unit, publish_message
from .scheduling.models import ScheduledTask

from courses.classes.models import Class
from discussion.models import Forum
from courses.certification.models import CertificateTemplate
from courses.events.models import Event

from celery.result import AsyncResult
from celery import Celery
app = Celery()


def schedule_task(instance, task):

    content_type = ContentType.objects.get_for_model(instance)
    if instance.release_date:

        st, created = ScheduledTask.objects.get_or_create(
            content_type=content_type,
            object_id=instance.id,
        )
        if created or st.release_date != instance.release_date:

            if st.task_hash:
                app.control.revoke(st.task_hash)

            # update release date in baquara aux model
            st.release_date = instance.release_date

            # create celery tasks
            st.task_hash = task.apply_async(args=[instance.id], eta=instance.release_date)
            st.save()
    else:
        try:
            # case when not published but there is no release date and there is prior task scheduled
            st = ScheduledTask.objects.get(
                content_type=content_type,
                object_id=instance.id,
            )
            app.control.revoke(st.task_hash)
            st.delete()
        except ScheduledTask.DoesNotExist:
            return


def create_or_update_events(instance, type):

    course_base_url = "/course/{course_slug}/intro/"
    event = None

    if instance.release_date:

        if type == 'course':
            event, created = Event.objects.get_or_create(
                course=instance,
                type='course_start',
            )

            event.title_en = 'Launch of {course} course'.format(
                course = instance.name_en,
            )
            event.text_en = 'Access the course <a href="{course_url}">{course}</a> here'.format(
                course = instance.name_en,
                course_url = course_base_url.format(course_slug=instance.slug),
            )

            event.title_es = 'Lanzamiento del curso {course}'.format(
                course = instance.name_es,
                course_url = course_base_url.format(course_slug=instance.slug),
            )
            event.text_es = 'Accede al curso <a href="{course_url}">{course}</a> aquí'.format(
                course = instance.name_es,
                course_url = course_base_url.format(course_slug=instance.slug),
            )

            event.title_pt_br = 'Lançamento do curso {course}'.format(
                course = instance.name_pt_br,
                course_url = course_base_url.format(course_slug=instance.slug),
            )
            event.text_pt_br = 'Acesse aqui o curso <a href="{course_url}">{course}</a>'.format(
                course = instance.name_pt_br,
                course_url = course_base_url.format(course_slug=instance.slug),
            )

        elif (type == 'course_lesson_start' and
                instance.course and
                instance.course.status == 'published' and
                instance.course.has_started):

            event, created = Event.objects.get_or_create(
                course=instance.course,
                lesson=instance,
                type='course_lesson_start',
            )

            event.title_en = 'Launch of a new module on course {course}'.format(
                course = instance.course.name_en,
            )
            event.text_en = 'Access the new module {lesson} from the course <a href="{course_url}">{course}</a> home'.format(
                lesson = instance.name_en,
                course = instance.course.name_en,
                course_url = course_base_url.format(course_slug=instance.course.slug),
            )

            event.title_es = 'Lanzamiento de un nuevo módulo en el curso {course}'.format(
                course = instance.course.name_es,
            )
            event.text_es = 'Accede al nuevo módulo {lesson} desde la página de inicio del curso <a href="{course_url}">{course}</a>'.format(
                lesson = instance.name_es,
                course = instance.course.name_es,
                course_url = course_base_url.format(course_slug=instance.slug),
            )

            event.title_pt_br = 'Lançamento de novo módulo no curso {course}'.format(
                course = instance.course.name_pt_br,
            )
            event.text_pt_br = 'Acesse o novo módulo {lesson} a partir da página inicial do curso <a href="{course_url}">{course}</a>'.format(
                lesson = instance.name_pt_br,
                course = instance.course.name_pt_br,
                course_url = course_base_url.format(course_slug=instance.course.slug),
            )

        elif (
                type == 'course_unit_start'and
                instance.lesson and
                instance.lesson.course and
                instance.lesson.course.status == 'published' and
                instance.lesson.course.has_started
                ):

            event, created = Event.objects.get_or_create(
                course=instance.lesson.course,
                lesson=instance.lesson,
                unit=instance,
                type='course_unit_start',
            )

            event.title_en = 'Launch of a new unit on course {course}'.format(
                course = instance.lesson.course.name_en,
            )
            event.text_en = 'Access the new unit {unit} from the course <a href="{course_url}">{course}</a> home'.format(
                unit = instance.title_en,
                course = instance.lesson.course.name_en,
                course_url = course_base_url.format(course_slug=instance.lesson.course.slug),
            )

            event.title_es = 'Lanzamiento de una nueva unidad en el curso {course}'.format(
                course = instance.lesson.course.name_es,
            )
            event.text_es = 'Accede al nueva undad {unit} desde la página de inicio del curso <a href="{course_url}">{course}</a>'.format(
                unit = instance.title_es,
                course = instance.lesson.course.name_es,
                course_url = course_base_url.format(course_slug=instance.lesson.course.slug),
            )

            event.title_pt_br = 'Lançamento de nova unidade no curso {course}'.format(
                course = instance.lesson.course.name_pt_br,
            )
            event.text_pt_br = 'Acesse a nova unidade {unit} a partir da página inicial do curso <a href="{course_url}">{course}</a>'.format(
                unit = instance.title_pt_br,
                course = instance.lesson.course.name_pt_br,
                course_url = course_base_url.format(course_slug=instance.lesson.course.slug),
            )

        # update release date in baquara aux model
        if event:
            event.release_date = instance.release_date
            event.save()

    else:
        # case when not published but there is no release date and there is prior event
        if type == 'course':
            # less DRY please
            try:
                event = Event.objects.get(
                    course=instance,
                    type='course_start',
                )
            except Event.DoesNotExist:
                pass
        if type == 'course_lesson_start':
            try:
                event = Event.objects.get(
                    course=instance.course,
                    lesson=instance,
                    type='course_lesson_start',
                )
            except Event.DoesNotExist:
                pass

        if type == 'course_unit_start':
            try:
                event = Event.objects.get(
                    course=instance.lesson.course,
                    lesson=instance.lesson,
                    unit=instance,
                    type='course_unit_start',
                )
            except Event.DoesNotExist:
                pass

        if event:
            event.delete()


def clear_scheduled_tasks(instance):

    content_type = ContentType.objects.get_for_model(instance)
    try:
        st = ScheduledTask.objects.get(content_type=content_type, object_id=instance.id)
        if st.task_hash:
            app.control.revoke(st.task_hash)
        st.delete()
    except ScheduledTask.DoesNotExist:
        pass


@receiver(post_save, sender=Lesson)
def post_lesson_created_or_updated(sender, instance, **kwargs):

    if instance.status != 'published':
        schedule_task(instance, publish_lesson)
        create_or_update_events(instance, 'course_lesson_start')
    else:
        clear_scheduled_tasks(instance)


@receiver(post_save, sender=Unit)
def post_unit_created_or_updated(sender, instance, **kwargs):

    if instance.status != 'published':
        schedule_task(instance, publish_unit)
        create_or_update_events(instance, 'course_unit_start')
    else:
        clear_scheduled_tasks(instance)


@receiver(post_save, sender=ProfessorMessage)
def post_professor_message_save(sender, instance: ProfessorMessage, **kwargs):

    if instance.status != 'published':
        schedule_task(instance, publish_message)
    else:
        clear_scheduled_tasks(instance)


@receiver(post_save, sender=Course)
def course_created_or_updated(sender, created, instance, **kwargs):

    if created:
        c = Class.objects.create(name=instance.name, course=instance)
        instance.default_class = c

        for type_tuple in CertificateTemplate.TYPES:
            type_name = type_tuple[0]

            ct = CertificateTemplate(course=instance, document_type=type_name, name='Base ' + instance.name)
            ct.save()

            if type_name == 'receipt':
                instance.default_class.receipt_template = ct
            elif type_name == 'certificate':
                instance.default_class.certificate_template = ct

        instance.default_class.save()

        forum = Forum.objects.create(
            title=instance.name,
            forum_type="course",
            )
        instance.forum = forum

        instance.save()
 
    else:
        forum = instance.forum

    title=instance.name
    forum.title_en = instance.name_en
    forum.title_es = instance.name_es
    forum.title_pt_br = instance.name_pt_br
    forum.is_public=instance.is_public
    forum.groups.set(instance.groups.all())
    forum.save()

    # the course has a different logic for the release date, as it must
    # create de event only when is in the published state
    if instance.start_date and instance.status == 'published':
        create_or_update_events(instance, 'course')

    if instance.status == 'draft':
        # TODO decide if delete all or only course_start type. If delete all,
        # then they has to be recriated when the status change to public again
        # in case there is some module or unit scheduled
        # instance.events.all().delete()
        instance.events.filter(type='course_start').delete()

@receiver(post_delete, sender=Course)
def create_or_update_course_forum(sender, instance, **kwargs):

    instance.forum.delete()

@receiver(email_confirmed)
def send_welcome_message(request, email_address,  **kwargs):
    
    ProfessorMessage.send_welcome_messages(email_address.user)
