from rest_framework import serializers
from actstream.models import Action
from courses.reports.serializers import UserStatsSerializer
from courses.serializers import SimpleUserSerializer

from django.contrib.auth.models import User
from django.contrib.auth import get_user_model
User = get_user_model()

from courses.models import Course, Unit
from courses.stats.models import AccessibleArea
from courses.classroom.models import Classroom
from courses.classroom.serializers import BasicClassroomSerializer

from discussion.models import Comment, Topic, Forum

from cards.models import Card

from courses_learning_objects.models import LearningObject, Answer

from datetime import datetime

class UserActionsSerializer(serializers.Serializer):
    verb = serializers.CharField(required=True, max_length=100)
    action_object_id = serializers.IntegerField(required=False)
    action_object_type = serializers.CharField(required=False, max_length=100)
    target_id = serializers.IntegerField(required=False)
    target_type = serializers.CharField(required=False, max_length=100)


class UserAccessSerializer(serializers.Serializer):
    area = serializers.CharField(required=True, max_length=100)
    target_id = serializers.IntegerField(required=False)
    target_type = serializers.CharField(required=False, max_length=100)


class GenericRelatedField(serializers.Field):
    def to_representation(self, value):
        if isinstance(value, User):
            return SimpleUserSerializer(value).data
        if isinstance(value, Classroom):
            return BasicClassroomSerializer(value).data
        return str(value)


class ActionSerializer(serializers.ModelSerializer):
    actor = GenericRelatedField(read_only=True)
    target = serializers.SerializerMethodField()
    action_object = GenericRelatedField(read_only=True)
    verb = GenericRelatedField(read_only=True)
    action_phrase = serializers.SerializerMethodField()
    complements = serializers.SerializerMethodField()
    timestamp = serializers.SerializerMethodField()

    class Meta:
        model = Action
        fields = ('actor', 'target', 'action_object', 'verb', 'action_phrase', 'timestamp', 'complements')

    def get_target(self, obj):
        if isinstance(obj.target, LearningObject):
            if not obj.target.unit:
                 return ''
            return obj.target.unit.lesson.course.name
        if isinstance(obj.target, Forum):
            return obj.target.title
        if isinstance(obj.target, Classroom):
            return obj.target.name
        if isinstance(obj.target, Topic):
            return obj.target.title
        return obj.target

        return obj.target

    def get_timestamp(self, obj):
        from django.utils import timezone
        tz_str = timezone.localtime(obj.timestamp)
        date, hour = tz_str.strftime('%d-%m-%Y %H:%M:%S').split()
        return '{} / {}'.format(date, hour)

    def get_object_type(self, obj, verb, lang):
        if isinstance(obj, Course):
            if lang == 'en':
                course = ' at course {}'
            if lang == 'pt-br':
                course = ' no curso {}'
            if lang == 'es':
                course = ' en el curso {}'
            return course.format(obj.name)
        if isinstance(obj, Classroom):
            if lang == 'en':
                classroom = ' in the classroom {}'
            if lang == 'pt-br':
                classrom = ' na sala {}'
            if lang == 'es':
                classroom = ' en la sala {}'
            return classroom.format(obj.name)
        if isinstance(obj, Topic):
            if verb == 'reacted':
                if lang == 'en':
                    topic = ' to the topic {}'
                if lang == 'pt-br':
                    topic = ' ao tópico {}'
                if lang == 'es':
                    topic = ' al tema {}'
            else:
                if lang == 'en':
                    topic = ' in the topic {}'
                if lang == 'pt-br':
                    topic = ' no tópico {}'
                if lang == 'es':
                    topic = ' en el tema {}'
            return topic.format(obj.title)
        if isinstance(obj, Forum):
            if lang == 'en':
                forum = ' in the forum {}'
            if lang == 'pt-br':
                forum = ' no fórum {}'
            if lang == 'es':
                forum = ' en el foro {}'
            return forum.format(obj.title)
        if isinstance(obj, AccessibleArea):
            return ' {}'.format(obj.name)
        if isinstance(obj, LearningObject):
            if obj.unit:
                unit = obj.unit.title
                step = obj.unit.lesson.name
                course = obj.unit.lesson.course.name
                if lang == 'en':
                    lo = ' to the activity {} in step {} of course {}'
                if lang == 'pt-br':
                    lo = ' a atividade {} na etapa {} do curso {}'
                if lang == 'es':
                    lo = ' a la actividad {} en el paso {} do curso {}'
                return lo.format(unit, step, course)
        if isinstance(obj, Card):
            if verb == 'reacted':
                if lang == 'en':
                    card = ' to the content {}'
                if lang == 'pt-br':
                    card = ' ao conteúdo {}'
                if lang == 'es':
                    card = ' al contenido {}'
            else:
                if lang == 'en':
                    card = ' the content {}'
                if lang == 'pt-br':
                    card = ' o conteúdo {}'
                if lang == 'es':
                    card = ' el contenido {}'
                card = ' o conteúdo {}'
            return card.format(obj.title)
        if isinstance(obj, Unit):
            if lang == 'en':
                unit = ' the unit {} in step {} of course {}'
            if lang == 'pt-br':
                unit = ' a unidade {} na etapa {} do curso {}'
            if lang == 'es':
                unit = ' a la unidad {} en el paso {} do curso {}'
            return unit.format(obj.title, obj.lesson, obj.lesson.course)
        return ''

    def get_actor_name(self, actor):
        if actor.name:
            return actor.name.split()[0]
        return actor.username

    def get_verb(self, verb, lang):
        verbs = {
            'access': {'en': 'accessed', 'pt-br': 'acessou', 'es': 'accedió'},
            'answered': {'en': 'answered', 'pt-br': 'respondeu', 'es': 'respondió'},
            'created': {'en': 'created', 'pt-br': 'criou', 'es': 'creado'},
            'reacted': {'en': 'reacted', 'pt-br': 'reagiu', 'es': 'reaccionó'},
            'created comment': {'en': 'commented', 'pt-br': 'comentou', 'es': 'comentó'}
        }
        return verbs[verb][lang]


    def get_action_phrase(self, obj):
        from django.utils.translation import get_language
        lang = get_language()

        action_phrase = ''

        action_phrase += self.get_actor_name(obj.actor)
        action_phrase += (' {}'.format(self.get_verb(obj.verb, lang)))

        if obj.action_object:
            action_phrase += self.get_object_type(obj.action_object, obj.verb, lang)
        if obj.target:
            action_phrase += self.get_object_type(obj.target, obj.verb, lang)

        return action_phrase

    def get_complements(self, obj):
        if obj.target:
            if isinstance(obj.target, LearningObject) and obj.target.unit:
                unit = 'a atividade {}'.format(obj.target.unit.title)
                step = 'na etapa {}'.format(obj.target.unit.lesson.name)
                course = 'do curso {}'.format(obj.target.unit.lesson.course.name)
                compl =  {'atividade': unit, 'etapa': step, 'curso': course}
                return compl

        if obj.action_object and obj.target:
            if isinstance(obj.action_object, Card) and isinstance(obj.target, Classroom):
                if obj.verb == 'reacted':
                    content = 'ao conteúdo {}'
                else:
                    content = 'o conteúdo {}'

                title = content.format(obj.action_object.title)
                classroom = 'na sala {}'.format(obj.target.name)
                compl = {'title': title, 'classroom': classroom, 'curso': ''}
                return compl

            if isinstance(obj.action_object, Topic) and isinstance(obj.target, Forum):
                if obj.verb == 'reacted':
                    topic = 'ao tópico {}'
                else:
                    topic = 'no tópico {}'

                forum = 'no fórum {}'.format(obj.target.title)
                compl = {'topic': topic.format(obj.action_object.title), 'forum': forum, 'curso': ''}
                return compl

        if obj.action_object:
            if isinstance(obj.action_object, Unit):
                unit = 'a unidade {}'.format(obj.action_object.title)
                step = 'na etapa {}'.format(obj.action_object.lesson)
                course = 'do curso {}'.format(obj.action_object.lesson.course)
                compl =  {'atividade': unit, 'etapa': step, 'curso': course}
                return compl
        return ''
