from django.db.models import Q
from django.db import IntegrityError
from django.contrib.auth import get_user_model
from django.http import HttpResponse

from rest_framework import views, viewsets, mixins, permissions, status
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticated
from rest_framework.decorators import action

from .models import (Course, CourseProfessor, CourseStudent, ProfessorMessage,
                     ProfessorMessageRead, Class, Lesson,
    CourseAuthor,
    )

from .serializers import (
        CourseSerializer,
        BasicCourseProfessorSerializer,
        MessageBasicSerializer,
        MessageSerializer,
        MessageReadSerializer,
        BasicLessonSerializer,
        LessonSerializer,
        CourseAuthorSerializer,
        CourseAuthorPictureSerializer,
    )

from courses.workspaces.serializers import CourseGroupSerializer

from .permissions import (
    IsProfessorCoordinatorOrAdminPermissionOrReadOnly,
    IsAssistantOrCoordinatorOrAdminOrRecipient,
    IsAdminOrReadOnly,
)


class CourseGroupViewSet(viewsets.ModelViewSet):

    model = Course
    queryset = Course.objects.all()
    serializer_class = CourseGroupSerializer

    def get_queryset(self):
        queryset = super(CourseGroupViewSet, self).get_queryset()
        course = self.request.query_params.get('id', None)

        if course:
            queryset = queryset.filter(groups__id = course)

        return queryset


class CourseViewSet(viewsets.ModelViewSet):

    model = Course
    queryset = Course.objects.all()
    serializer_class = CourseSerializer
    lookup_field = 'id'
    filter_fields = ('slug', 'home_published',)
    permission_classes = (IsProfessorCoordinatorOrAdminPermissionOrReadOnly,)

    def get_queryset(self):
        queryset = super(CourseViewSet, self).get_queryset()

        if not self.request.user.is_authenticated:
            return queryset.filter(is_public=True, status='published').prefetch_related('professors')

        if not self.request.user.is_superuser:
            role = self.request.query_params.get('role', None)
            if role:
                queryset = queryset.filter(
                    course_professors__role=role,
                    course_professors__user=self.request.user
                ).prefetch_related('professors')

            # Course must have status status published and (be public or share groups with user)
            #query = Q(
            #    Q(status='published') & (
            #        Q(is_public=True, ) | Q(groups__in=self.request.user.groups.all(), status='published')
            #    )
            #)
            from django.utils.translation import get_language
            current_language = get_language()
            query = Q(Q(status='published', lang__contains=[current_language]) & Q(Q(is_public=True, ) | Q(groups__in=self.request.user.groups.all(), status='published')))

            if (CourseProfessor.objects.filter(user=self.request.user, role='coordinator').exists()):
                query.add(Q(
                    course_professors__role='coordinator',
                    course_professors__user=self.request.user
                ), Q.OR)

            queryset = queryset.filter(query)

        queryset = queryset.prefetch_related('professors')

        return queryset.distinct()

    def post(self, request, **kwargs):
        course = self.get_object()

        serializer = CourseSerializer(course, request.data)

        if serializer.is_valid():
            serializer.save()
            return Response(status=200)
        else:
            return Response(serializer.errors, status=400)

    def metadata(self, request):
        data = super().metadata(request)
        if data.get('actions'):
            data.get('actions').get('POST').get('status').update({'choices': dict(Course.STATES[1:])})
        return data


class MyCoursesViewSet(viewsets.ModelViewSet):
    model = Course
    queryset = Course.objects.all()
    serializer_class = CourseSerializer
    permission_classes = (IsProfessorCoordinatorOrAdminPermissionOrReadOnly,)

    def get_queryset(self):
        queryset = super(MyCoursesViewSet, self).get_queryset()
        user = self.request.user
        course_students = CourseStudent.objects.filter(user__id=user.id).prefetch_related('course')
        courses_id = list(course_students.values_list('course__id', flat=True))
        queryset = self.queryset.filter(id__in=courses_id)

        return queryset


class CourseBySlugViewSet(viewsets.ModelViewSet):
    model = Course
    queryset = Course.objects.all()
    serializer_class = CourseSerializer
    permission_classes = (IsAuthenticated,)
    lookup_field = 'slug'


class BaseCourseProfessorViewSet(viewsets.ModelViewSet):

    model = CourseProfessor
    queryset = CourseProfessor.objects.all()
    # lookup_field = 'id'
    filter_fields = (
        'course',
        'user',
        'role',
    )
    serializer_class = BasicCourseProfessorSerializer
    permission_classes = (IsProfessorCoordinatorOrAdminPermissionOrReadOnly, )


class MessageMixin():

   def get_queryset(self):
        queryset = super().get_queryset()

        # Filter only for the user course, in case the user is admin.
        my_messages = self.request.query_params.get('my_messages', None)
        if not self.request.user.is_superuser or my_messages:
            query = Q(
                Q(users=self.request.user)
                | Q(groups__in=self.request.user.groups.all())
                | Q(sender=self.request.user)
            ) & Q( status='published')
            queryset = queryset.filter(query).distinct()

        classroom = self.request.query_params.get('classroom', None)
        if classroom:
            queryset = queryset.filter(classrooms=classroom)

        unread = self.request.query_params.get('unread', None)
        if unread:
            # Exclude read messages
            queryset = queryset.exclude(read_status__is_read=True, read_status__user=self.request.user)

        queryset = queryset.order_by('-release_date').order_by('-date')

        limit_to = self.request.query_params.get('limit_to', None)
        if limit_to:
            queryset = queryset[:int(limit_to)]

        return queryset


class MessageBasicViewSet(MessageMixin, viewsets.ReadOnlyModelViewSet):
    model = ProfessorMessage
    queryset = ProfessorMessage.objects.all()
    filter_fields = ('course', 'classrooms', 'status', 'type',)
    serializer_class = MessageBasicSerializer
    permission_classes = (IsAssistantOrCoordinatorOrAdminOrRecipient, )

    @action(detail=False)
    def unread_count(self, request):
        # Count unread messages
        messages = ProfessorMessage.objects.filter(
            users=self.request.user,
        ).exclude(
            read_status__is_read=True,
            read_status__user=self.request.user,
        )
        # return Response(messages.count())
        return Response({'count': messages.count()})


class MessageViewSet(MessageMixin, viewsets.ModelViewSet):
    model = ProfessorMessage
    queryset = ProfessorMessage.objects.all()
    filter_fields = ('course', 'classrooms', 'status', 'type',)
    serializer_class = MessageSerializer
    permission_classes = (IsAssistantOrCoordinatorOrAdminOrRecipient, )

    def perform_create(self, serializer):
        # Ensure the sender is set on creation
        obj = serializer.save(sender=self.request.user)

    def get_queryset(self):
        queryset = super().get_queryset()

        if not self.request.user.is_superuser:
            # if the user is coordinator, he see send messages to all classes on the course
            queryset = queryset.filter(
                course__course_professors__user=self.request.user,
                course__course_professors__role='coordinator'
            )

            queryset = queryset.filter(classes__assistants=self.request.user)

        return queryset.prefetch_related('classrooms', 'classes', 'users', 'groups',).select_related('course', 'sender')

    @action(detail=False)
    def count_message(self, request):
        # Count unread messages
        messages = ProfessorMessage.objects.filter(
            users=self.request.user,
        ).exclude(
            read_status__is_read=True,
            read_status__user=self.request.user,
        )
        return Response({'count': messages.count()})

    @action(detail=True)
    def send(self, request, pk=None):
        message = self.get_object()
        user = request.user

        if message.status == 'published':
            return Response(
                {'error': 'This message was already sent before and connot be sent twice.'},
                status=400,
            )

        if (user.is_superuser and message.is_active_welcome and message.type == 'welcome'):
            message.send()
            return Response({'sucess': 'Messsage defined as welcome message!'})

        if not (message.is_broadcast or message.groups.exists() or message.recipients.exists() or message.classes.exists()):
            return Response(
                {'error': 'Cannot send a message with no recipient.'},
                status=400,
            )

        # General message, only allowed for admins
        if (user.is_superuser and (
                message.is_broadcast or 
                message.groups.exists() or 
                message.recipients.exists() or
                message.classes.exists())
        ):
            message.send()
            return Response({'sucess': 'Messsage sent!'})
        
        # Course message, allowed for Course Coordinator and class assistant to send to their classes
        if (message.course and message.classes.exists()) and not (message.is_broadcast or message.groups.exists()):
            allowed = False
            # if the user is coordinator, he can send messages to all classes
            if message.course.is_course_coordinator(request.user):
                allowed = True
            
            # if the user is assistant, he can send messages only to hin classes
            if message.course.is_course_assistant(request.user):
                for klass in message.classes:
                    if request.user not in message.classes.assistants.all():
                        allowed = False
            if allowed:
                message.send()
                return Response({'sucess': 'Messsage sent!'})

        return Response(
                {'error': 'This user is not allowed to send this message'},
                status=403,
            )


class MessageReadViewSet(viewsets.ModelViewSet):
    model = ProfessorMessageRead
    lookup_field = 'message'
    serializer_class = MessageReadSerializer
    permission_classes = (IsAuthenticated,)

    def perform_create(self, serializer):
        try:
            read_info = serializer.save(user=self.request.user, is_read=True)
        except IntegrityError as e:
            read_info = ProfessorMessageRead.objects.get(message=self.request.data['message'], user=self.request.user)
            read_info.is_read = True
            read_info.save()

    def get_queryset(self):
        queryset = ProfessorMessageRead.objects.filter(user=self.request.user).order_by('-id')

        limit_to = self.request.query_params.get('limit_to', None)
        if limit_to:
            queryset = queryset[:int(limit_to)]

        return queryset


class BasicLessonViewSet(viewsets.ModelViewSet):
    model = Lesson
    queryset = Lesson.objects.all()
    serializer_class = BasicLessonSerializer
    ordering = ('position',)
    filter_fields = ('course', 'status',)


class LessonViewSet(viewsets.ModelViewSet):
    model = Lesson
    queryset = Lesson.objects.all()
    serializer_class = LessonSerializer
    ordering = ('position',)

    def get_queryset(self):
        queryset = super(LessonViewSet, self).get_queryset()
        course_id = self.kwargs['course_id']
        return queryset.filter(status='published', units__status='published', course_id=course_id)


class CourseAuthorViewSet(viewsets.ModelViewSet):
    model = CourseAuthor
    queryset = CourseAuthor.objects.all()
    lookup_field = 'id'
    filter_fields = ('course', 'user',)
    serializer_class = CourseAuthorSerializer
    permission_classes = (IsProfessorCoordinatorOrAdminPermissionOrReadOnly, )

    def pre_save(self, obj):
        # Verify if current user is coordinator. The has_object_permission method is not called when creating objects,
        # so we call it explicitly here. See: https://github.com/tomchristie/django-rest-framework/issues/1103
        self.check_object_permissions(self.request, obj)
        return super(CourseAuthorViewSet, self).pre_save(obj)


class CoursePictureUploadViewSet(viewsets.ModelViewSet):
    model = CourseAuthor
    queryset = CourseAuthor.objects.all()
    lookup_field = 'id'
    serializer_class = CourseAuthorPictureSerializer

    def post(self, request, **kwargs):
        course = self.get_object()
        serializer = self.get_serializer(course, request.FILES)

        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=200)
        else:
            return Response(serializer.errors, status=400)


# class CourseGroupViewSet(viewsets.ModelViewSet):
#     queryset = Course.objects.all()
#     serializer_class = CourseGroupSerializer
#     permission_classes = (IsAuthenticated,)

#     def update(self, request, *args, **kwargs):
#         groups = request.data.get('groups', None)
#         course_slug = request.data.get('slug', None)
#         instance = Course.objects.get(slug=course_slug)
#         if groups and type(groups[0]) == int:
#             curr_groups = set(list(instance.groups.all().values_list('id', flat=True)))
#             remove_groups = curr_groups - set(groups)
#             add_groups = set(groups) - curr_groups

#             for g in add_groups:
#                 instance.groups.add(g)

#             for g in remove_groups:
#                 instance.groups.remove(g)

#             instance.save()
#             serializer = CourseGroupSerializer(instance)
#             return Response(serializer.data, status=status.HTTP_200_OK)
#         else:
#             return Response({'error' : 'no group set'},
#                             status=status.HTTP_400_BAD_REQUEST)
